//
//  Brahe
//  A heterogenous library of mathematical and scientific functions
//
//  mathtools.h
//
//  Copyright 2006, 2007 Scott Robert Ladd
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//      Free Software Foundation, Inc.
//      59 Temple Place - Suite 330
//      Boston, MA 02111-1307, USA.
//
//  For more information on this software package, please visit
//  Scott's web site, Coyote Gulch Productions, at:
//
//      http://www.coyotegulch.com
//
//  Closed-source licenses are available from the author at reasonable
//  prices.
//

#if !defined(LIBBRAHE_MATHTOOLS_H)
#define LIBBRAHE_MATHTOOLS_H

#include <stddef.h>
#include <math.h>
#include <limits.h>
#include <float.h>

#if defined(__cplusplus)
extern "C" {
#endif

#if defined(_MSC_VER)
#pragma warning (disable: 4244 4267)
#if !defined(__cplusplus)
typedef char bool;
static const int true  = 1;
static const int false = 0;
#endif
typedef unsigned __int64 uint64_t;
typedef __int64          int64_t;
typedef unsigned __int32 uint32_t;
typedef __int32          int32_t;
typedef unsigned __int16 uint16_t;
typedef __int16          int16_t;
typedef unsigned __int8  uint8_t;
typedef __int8           int8_t;
#else
// ISO C standard compilers
#include <stdbool.h>
#include <stdint.h>
#endif

//! Round to nearest value
/*!
    Rounds a value to nearest integer, rounding to even for exact fractions of 0.5.
*/
double brahe_round_nearest(const double x);

// Set number of significant digits in a floating-point value
/*!
    Returns the given value rounded for the number of specified decimal
    digits of precision.
    \param x - Original value to be rounded
    \param n - Number of significant digits
    \return The value <i>x</i> rounded to <i>n</i> digits of precision
*/
double brahe_sigdig(const double x, const uint16_t n);

// Lowest common multiple
/*!
    Calculates the lowest common multiple for two values.
*/
uint64_t brahe_lcm(const uint64_t x, const uint64_t y);

// Greatest common factor (denominator)
/*!
    Calculates the greatest common factor for two values.
*/
uint64_t brahe_gcf(uint64_t x, uint64_t y);

// Factor a number
/*!
    Calculates the prime factors of a number.
*/
uint64_t * brahe_factor(const uint64_t x);

//! Hyperbolic arcsine
/*!
    Calculates sinh<sup>-1</sup>.
*/
double brahe_asinh(const double x);

//! Hyperbolic arccosine
/*!
    Calculates cosh<sup>-1</sup>.
*/
double brahe_acosh(const double x);

//! Hyperbolic arctangent
/*!
    Calculates tanh<sup>-1</sup>.
*/
double brahe_atanh(const double x);

#if defined(__cplusplus)
}
#endif

#endif
